@@ -34,12 +34,14 @@ Main code for awsenergylabelerlib.
 import logging
 
 from cachetools import cached, TTLCache
+
+from .awsenergylabelerlibexceptions import UnsupportedZoneType
 from .configuration import (ACCOUNT_THRESHOLDS,
-                            LANDING_ZONE_THRESHOLDS,
+                            ZONE_THRESHOLDS,
                             DEFAULT_SECURITY_HUB_FILTER,
                             DEFAULT_SECURITY_HUB_FRAMEWORKS)
-from .entities import SecurityHub, LandingZone
-from .schemas import account_thresholds_schema, landing_zone_thresholds_schema
+from .entities import SecurityHub, AuditZone, OrganizationsZone
+from .schemas import account_thresholds_schema, zone_thresholds_schema
 
 __author__ = 'Costas Tyfoxylos <ctyfoxylos@schubergphilis.com>'
 __docformat__ = '''google'''
@@ -58,41 +60,47 @@ LOGGER_BASENAME = '''awsenergylabelerlib'''
 LOGGER = logging.getLogger(LOGGER_BASENAME)
 LOGGER.addHandler(logging.NullHandler())
 
+SUPPORTED_ZONE_TYPES = {'organizations_zone': OrganizationsZone,
+                        'audit_zone': AuditZone}
+
 
 class EnergyLabeler:  # pylint: disable=too-many-arguments,  too-many-instance-attributes
-    """Labeling accounts and landing zone based on findings and label configurations."""
+    """Labeling accounts and zone based on findings and label configurations."""
 
     # pylint: disable=dangerous-default-value
     def __init__(self,
-                 landing_zone_name,
+                 zone_name,
                  region=None,
                  account_thresholds=ACCOUNT_THRESHOLDS,
-                 landing_zone_thresholds=LANDING_ZONE_THRESHOLDS,
+                 zone_thresholds=ZONE_THRESHOLDS,
                  security_hub_filter=DEFAULT_SECURITY_HUB_FILTER,
                  frameworks=DEFAULT_SECURITY_HUB_FRAMEWORKS,
                  allowed_account_ids=None,
                  denied_account_ids=None,
                  allowed_regions=None,
-                 denied_regions=None):
+                 denied_regions=None,
+                 zone_type='organizations_zone'):
         self._logger = logging.getLogger(f'{LOGGER_BASENAME}.{self.__class__.__name__}')
         self.account_thresholds = account_thresholds_schema.validate(account_thresholds)
-        self.landing_zone_thresholds = landing_zone_thresholds_schema.validate(landing_zone_thresholds)
+        self.zone_thresholds = zone_thresholds_schema.validate(zone_thresholds)
+        zone_type_obj = self._validate_zone_type(zone_type)
         self._security_hub_filter = security_hub_filter
         self._frameworks = SecurityHub.validate_frameworks(frameworks)
-        self._landing_zone = self._initialize_landing_zone(landing_zone_name, allowed_account_ids, denied_account_ids)
         self._security_hub = self._initialize_security_hub(region, allowed_regions, denied_regions)
         self._account_labels_counter = None
         self._query_filter = None
-        self._landing_zone_energy_label = None
+        self._zone_energy_label = None
         self._labeled_accounts_energy_label = None
-        self._landing_zone_labeled_accounts = None
+        self._zone_labeled_accounts = None
+        self._zone = zone_type_obj(zone_name, region, allowed_account_ids, denied_account_ids, zone_thresholds,
+                                   account_thresholds)
 
-    def _initialize_landing_zone(self, name, allowed_account_ids, denied_account_ids):
-        return LandingZone(name,
-                           self.landing_zone_thresholds,
-                           self.account_thresholds,
-                           allowed_account_ids,
-                           denied_account_ids)
+    @staticmethod
+    def _validate_zone_type(zone_type):
+        zone_type = zone_type.lower()
+        if zone_type not in SUPPORTED_ZONE_TYPES:
+            raise UnsupportedZoneType(zone_type)
+        return SUPPORTED_ZONE_TYPES[zone_type]
 
     @staticmethod
     def _initialize_security_hub(region, allowed_regions, denied_regions):
@@ -102,7 +110,7 @@ class EnergyLabeler:  # pylint: disable=too-many-arguments,  too-many-instance-a
 
     @property
     def initialized_security_hub_query_filter(self):
-        """Calculates and saves the security hub query filter based on the configuration of the landing zone args.
+        """Calculates and saves the security hub query filter based on the configuration of the zone args.
 
         Returns:
             query_filter (dict): The query filter constructed and cached.
@@ -110,8 +118,8 @@ class EnergyLabeler:  # pylint: disable=too-many-arguments,  too-many-instance-a
         """
         if self._query_filter is None:
             self._query_filter = SecurityHub.calculate_query_filter(self._security_hub_filter,
-                                                                    self._landing_zone.allowed_account_ids,
-                                                                    self._landing_zone.denied_account_ids,
+                                                                    self._zone.allowed_account_ids,
+                                                                    self._zone.denied_account_ids,
                                                                     self._frameworks)
             self._logger.debug(f'Calculated query {self._query_filter} to execute on security hub.')
         return self._query_filter
@@ -122,9 +130,9 @@ class EnergyLabeler:  # pylint: disable=too-many-arguments,  too-many-instance-a
         return self._frameworks
 
     @property
-    def landing_zone(self):
-        """Landing zone."""
-        return self._landing_zone
+    def zone(self):
+        """Zone."""
+        return self._zone
 
     @property
     def security_hub(self):
@@ -139,25 +147,34 @@ class EnergyLabeler:  # pylint: disable=too-many-arguments,  too-many-instance-a
         return self.security_hub.filter_findings_by_frameworks(findings, self._frameworks)
 
     @property
-    def landing_zone_energy_label(self):
-        """Energy label of the landing zone."""
-        if self._landing_zone_energy_label is None:
-            self._logger.debug(f'Landing zone accounts labeled are {len(self._landing_zone.accounts_to_be_labeled)}')
-            self._landing_zone_energy_label = self._landing_zone.get_energy_label(self.security_hub_findings)
-        return self._landing_zone_energy_label
+    def zone_energy_label(self):
+        """Energy label of the zone."""
+        if self._zone_energy_label is None:
+            self._logger.debug(f'Zone accounts labeled are {len(self._zone.accounts_to_be_labeled)}')
+            self._zone_energy_label = self._zone.get_energy_label(self.security_hub_findings)
+        return self._zone_energy_label
 
     @property
     def labeled_accounts_energy_label(self):
         """Energy label of the labeled accounts."""
         if self._labeled_accounts_energy_label is None:
-            self._labeled_accounts_energy_label = self._landing_zone.get_energy_label_of_targeted_accounts(
+            self._labeled_accounts_energy_label = self._zone.get_energy_label_of_targeted_accounts(
                 self.security_hub_findings)
         return self._labeled_accounts_energy_label
 
     @property
-    def landing_zone_labeled_accounts(self):
-        """The landing zone labeled account objects."""
-        if self._landing_zone_labeled_accounts is None:
-            self._landing_zone_labeled_accounts = self._landing_zone.get_labeled_targeted_accounts(
+    def zone_labeled_accounts(self):
+        """The zone labeled account objects."""
+        if self._zone_labeled_accounts is None:
+            self._zone_labeled_accounts = self._zone.get_labeled_targeted_accounts(
                 self.security_hub_findings)
-        return self._landing_zone_labeled_accounts
+        return self._zone_labeled_accounts
+
+    def get_zone_energy_label_for_findings_by_query(self, findings_query):
+        """Calculates an energy label on an arbitrary security hub findings query."""
+        findings = self.security_hub.get_findings(findings_query)
+        return self._zone.get_energy_label(findings)
+
+    def get_zone_energy_label_for_findings(self, findings):
+        """Calculates an energy label on provided security hub findings."""
+        return self._zone.get_energy_label(findings)
